/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package modelo.controle;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import modelo.Compra;
import modelo.CompraPK;
import modelo.Produto;
import modelo.PedidoCompra;
import modelo.controle.exceptions.NonexistentEntityException;
import modelo.controle.exceptions.PreexistingEntityException;

/**
 *
 * @author willian
 */
public class CompraJpaController {

    public CompraJpaController() {
        emf = Persistence.createEntityManagerFactory("EstoquePU");
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(Compra compra) throws PreexistingEntityException, Exception {
        if (compra.getCompraPK() == null) {
            compra.setCompraPK(new CompraPK());
        }
        compra.getCompraPK().setProdutoId(compra.getProduto().getId());
        compra.getCompraPK().setPedidoCompraid(compra.getPedidoCompra().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Produto produto = compra.getProduto();
            if (produto != null) {
                produto = em.getReference(produto.getClass(), produto.getId());
                compra.setProduto(produto);
            }
            PedidoCompra pedidoCompra = compra.getPedidoCompra();
            if (pedidoCompra != null) {
                pedidoCompra = em.getReference(pedidoCompra.getClass(), pedidoCompra.getId());
                compra.setPedidoCompra(pedidoCompra);
            }
            em.persist(compra);
            if (produto != null) {
                produto.getCompraCollection().add(compra);
                produto = em.merge(produto);
            }
            if (pedidoCompra != null) {
                pedidoCompra.getCompraCollection().add(compra);
                pedidoCompra = em.merge(pedidoCompra);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findCompra(compra.getCompraPK()) != null) {
                throw new PreexistingEntityException("Compra " + compra + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(Compra compra) throws NonexistentEntityException, Exception {
        compra.getCompraPK().setProdutoId(compra.getProduto().getId());
        compra.getCompraPK().setPedidoCompraid(compra.getPedidoCompra().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Compra persistentCompra = em.find(Compra.class, compra.getCompraPK());
            Produto produtoOld = persistentCompra.getProduto();
            Produto produtoNew = compra.getProduto();
            PedidoCompra pedidoCompraOld = persistentCompra.getPedidoCompra();
            PedidoCompra pedidoCompraNew = compra.getPedidoCompra();
            if (produtoNew != null) {
                produtoNew = em.getReference(produtoNew.getClass(), produtoNew.getId());
                compra.setProduto(produtoNew);
            }
            if (pedidoCompraNew != null) {
                pedidoCompraNew = em.getReference(pedidoCompraNew.getClass(), pedidoCompraNew.getId());
                compra.setPedidoCompra(pedidoCompraNew);
            }
            compra = em.merge(compra);
            if (produtoOld != null && !produtoOld.equals(produtoNew)) {
                produtoOld.getCompraCollection().remove(compra);
                produtoOld = em.merge(produtoOld);
            }
            if (produtoNew != null && !produtoNew.equals(produtoOld)) {
                produtoNew.getCompraCollection().add(compra);
                produtoNew = em.merge(produtoNew);
            }
            if (pedidoCompraOld != null && !pedidoCompraOld.equals(pedidoCompraNew)) {
                pedidoCompraOld.getCompraCollection().remove(compra);
                pedidoCompraOld = em.merge(pedidoCompraOld);
            }
            if (pedidoCompraNew != null && !pedidoCompraNew.equals(pedidoCompraOld)) {
                pedidoCompraNew.getCompraCollection().add(compra);
                pedidoCompraNew = em.merge(pedidoCompraNew);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                CompraPK id = compra.getCompraPK();
                if (findCompra(id) == null) {
                    throw new NonexistentEntityException("The compra with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(CompraPK id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Compra compra;
            try {
                compra = em.getReference(Compra.class, id);
                compra.getCompraPK();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The compra with id " + id + " no longer exists.", enfe);
            }
            Produto produto = compra.getProduto();
            if (produto != null) {
                produto.getCompraCollection().remove(compra);
                produto = em.merge(produto);
            }
            PedidoCompra pedidoCompra = compra.getPedidoCompra();
            if (pedidoCompra != null) {
                pedidoCompra.getCompraCollection().remove(compra);
                pedidoCompra = em.merge(pedidoCompra);
            }
            em.remove(compra);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<Compra> findCompraEntities() {
        return findCompraEntities(true, -1, -1);
    }

    public List<Compra> findCompraEntities(int maxResults, int firstResult) {
        return findCompraEntities(false, maxResults, firstResult);
    }

    private List<Compra> findCompraEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from Compra as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public Compra findCompra(CompraPK id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(Compra.class, id);
        } finally {
            em.close();
        }
    }

    public int getCompraCount() {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select count(o) from Compra as o");
            return ((Long) q.getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }

}
