/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package modelo.controle;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import modelo.Fornece;
import modelo.FornecePK;
import modelo.Produto;
import modelo.Fornecedor;
import modelo.controle.exceptions.NonexistentEntityException;
import modelo.controle.exceptions.PreexistingEntityException;

/**
 *
 * @author willian
 */
public class ForneceJpaController {

    public ForneceJpaController() {
        emf = Persistence.createEntityManagerFactory("EstoquePU");
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(Fornece fornece) throws PreexistingEntityException, Exception {
        if (fornece.getFornecePK() == null) {
            fornece.setFornecePK(new FornecePK());
        }
        fornece.getFornecePK().setProdutoid(fornece.getProduto().getId());
        fornece.getFornecePK().setFornecedorid(fornece.getFornecedor().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Produto produto = fornece.getProduto();
            if (produto != null) {
                produto = em.getReference(produto.getClass(), produto.getId());
                fornece.setProduto(produto);
            }
            Fornecedor fornecedor = fornece.getFornecedor();
            if (fornecedor != null) {
                fornecedor = em.getReference(fornecedor.getClass(), fornecedor.getId());
                fornece.setFornecedor(fornecedor);
            }
            em.persist(fornece);
            if (produto != null) {
                produto.getForneceCollection().add(fornece);
                produto = em.merge(produto);
            }
            if (fornecedor != null) {
                fornecedor.getForneceCollection().add(fornece);
                fornecedor = em.merge(fornecedor);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findFornece(fornece.getFornecePK()) != null) {
                throw new PreexistingEntityException("Fornece " + fornece + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(Fornece fornece) throws NonexistentEntityException, Exception {
        fornece.getFornecePK().setProdutoid(fornece.getProduto().getId());
        fornece.getFornecePK().setFornecedorid(fornece.getFornecedor().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Fornece persistentFornece = em.find(Fornece.class, fornece.getFornecePK());
            Produto produtoOld = persistentFornece.getProduto();
            Produto produtoNew = fornece.getProduto();
            Fornecedor fornecedorOld = persistentFornece.getFornecedor();
            Fornecedor fornecedorNew = fornece.getFornecedor();
            if (produtoNew != null) {
                produtoNew = em.getReference(produtoNew.getClass(), produtoNew.getId());
                fornece.setProduto(produtoNew);
            }
            if (fornecedorNew != null) {
                fornecedorNew = em.getReference(fornecedorNew.getClass(), fornecedorNew.getId());
                fornece.setFornecedor(fornecedorNew);
            }
            fornece = em.merge(fornece);
            if (produtoOld != null && !produtoOld.equals(produtoNew)) {
                produtoOld.getForneceCollection().remove(fornece);
                produtoOld = em.merge(produtoOld);
            }
            if (produtoNew != null && !produtoNew.equals(produtoOld)) {
                produtoNew.getForneceCollection().add(fornece);
                produtoNew = em.merge(produtoNew);
            }
            if (fornecedorOld != null && !fornecedorOld.equals(fornecedorNew)) {
                fornecedorOld.getForneceCollection().remove(fornece);
                fornecedorOld = em.merge(fornecedorOld);
            }
            if (fornecedorNew != null && !fornecedorNew.equals(fornecedorOld)) {
                fornecedorNew.getForneceCollection().add(fornece);
                fornecedorNew = em.merge(fornecedorNew);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                FornecePK id = fornece.getFornecePK();
                if (findFornece(id) == null) {
                    throw new NonexistentEntityException("The fornece with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(FornecePK id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Fornece fornece;
            try {
                fornece = em.getReference(Fornece.class, id);
                fornece.getFornecePK();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The fornece with id " + id + " no longer exists.", enfe);
            }
            Produto produto = fornece.getProduto();
            if (produto != null) {
                produto.getForneceCollection().remove(fornece);
                produto = em.merge(produto);
            }
            Fornecedor fornecedor = fornece.getFornecedor();
            if (fornecedor != null) {
                fornecedor.getForneceCollection().remove(fornece);
                fornecedor = em.merge(fornecedor);
            }
            em.remove(fornece);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<Fornece> findForneceEntities() {
        return findForneceEntities(true, -1, -1);
    }

    public List<Fornece> findForneceEntities(int maxResults, int firstResult) {
        return findForneceEntities(false, maxResults, firstResult);
    }

    private List<Fornece> findForneceEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from Fornece as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public Fornece findFornece(FornecePK id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(Fornece.class, id);
        } finally {
            em.close();
        }
    }

    public int getForneceCount() {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select count(o) from Fornece as o");
            return ((Long) q.getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }

}
