/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package modelo.controle;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import modelo.PedidoProducao;
import modelo.Produto;
import modelo.Funcionario;
import modelo.controle.exceptions.NonexistentEntityException;
import modelo.controle.exceptions.PreexistingEntityException;

/**
 *
 * @author willian
 */
public class PedidoProducaoJpaController {

    public PedidoProducaoJpaController() {
        emf = Persistence.createEntityManagerFactory("EstoquePU");
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(PedidoProducao pedidoProducao) throws PreexistingEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Produto produto = pedidoProducao.getProduto();
            if (produto != null) {
                produto = em.getReference(produto.getClass(), produto.getId());
                pedidoProducao.setProduto(produto);
            }
            Funcionario funcionario = pedidoProducao.getFuncionario();
            if (funcionario != null) {
                funcionario = em.getReference(funcionario.getClass(), funcionario.getId());
                pedidoProducao.setFuncionario(funcionario);
            }
            em.persist(pedidoProducao);
            if (produto != null) {
                produto.getPedidoProducaoCollection().add(pedidoProducao);
                produto = em.merge(produto);
            }
            if (funcionario != null) {
                funcionario.getPedidoProducaoCollection().add(pedidoProducao);
                funcionario = em.merge(funcionario);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findPedidoProducao(pedidoProducao.getId()) != null) {
                throw new PreexistingEntityException("PedidoProducao " + pedidoProducao + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(PedidoProducao pedidoProducao) throws NonexistentEntityException, Exception {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            PedidoProducao persistentPedidoProducao = em.find(PedidoProducao.class, pedidoProducao.getId());
            Produto produtoOld = persistentPedidoProducao.getProduto();
            Produto produtoNew = pedidoProducao.getProduto();
            Funcionario funcionarioOld = persistentPedidoProducao.getFuncionario();
            Funcionario funcionarioNew = pedidoProducao.getFuncionario();
            if (produtoNew != null) {
                produtoNew = em.getReference(produtoNew.getClass(), produtoNew.getId());
                pedidoProducao.setProduto(produtoNew);
            }
            if (funcionarioNew != null) {
                funcionarioNew = em.getReference(funcionarioNew.getClass(), funcionarioNew.getId());
                pedidoProducao.setFuncionario(funcionarioNew);
            }
            pedidoProducao = em.merge(pedidoProducao);
            if (produtoOld != null && !produtoOld.equals(produtoNew)) {
                produtoOld.getPedidoProducaoCollection().remove(pedidoProducao);
                produtoOld = em.merge(produtoOld);
            }
            if (produtoNew != null && !produtoNew.equals(produtoOld)) {
                produtoNew.getPedidoProducaoCollection().add(pedidoProducao);
                produtoNew = em.merge(produtoNew);
            }
            if (funcionarioOld != null && !funcionarioOld.equals(funcionarioNew)) {
                funcionarioOld.getPedidoProducaoCollection().remove(pedidoProducao);
                funcionarioOld = em.merge(funcionarioOld);
            }
            if (funcionarioNew != null && !funcionarioNew.equals(funcionarioOld)) {
                funcionarioNew.getPedidoProducaoCollection().add(pedidoProducao);
                funcionarioNew = em.merge(funcionarioNew);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                Integer id = pedidoProducao.getId();
                if (findPedidoProducao(id) == null) {
                    throw new NonexistentEntityException("The pedidoProducao with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(Integer id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            PedidoProducao pedidoProducao;
            try {
                pedidoProducao = em.getReference(PedidoProducao.class, id);
                pedidoProducao.getId();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The pedidoProducao with id " + id + " no longer exists.", enfe);
            }
            Produto produto = pedidoProducao.getProduto();
            if (produto != null) {
                produto.getPedidoProducaoCollection().remove(pedidoProducao);
                produto = em.merge(produto);
            }
            Funcionario funcionario = pedidoProducao.getFuncionario();
            if (funcionario != null) {
                funcionario.getPedidoProducaoCollection().remove(pedidoProducao);
                funcionario = em.merge(funcionario);
            }
            em.remove(pedidoProducao);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<PedidoProducao> findPedidoProducaoEntities() {
        return findPedidoProducaoEntities(true, -1, -1);
    }

    public List<PedidoProducao> findPedidoProducaoEntities(int maxResults, int firstResult) {
        return findPedidoProducaoEntities(false, maxResults, firstResult);
    }

    private List<PedidoProducao> findPedidoProducaoEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from PedidoProducao as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public PedidoProducao findPedidoProducao(Integer id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(PedidoProducao.class, id);
        } finally {
            em.close();
        }
    }

    public int getPedidoProducaoCount() {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select count(o) from PedidoProducao as o");
            return ((Long) q.getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }

}
