/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package modelo.controle;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityNotFoundException;
import modelo.Produto;
import modelo.PedidoVenda;
import modelo.Venda;
import modelo.VendaPK;
import modelo.controle.exceptions.NonexistentEntityException;
import modelo.controle.exceptions.PreexistingEntityException;

/**
 *
 * @author willian
 */
public class VendaJpaController {

    public VendaJpaController() {
        emf = Persistence.createEntityManagerFactory("EstoquePU");
    }
    private EntityManagerFactory emf = null;

    public EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    public void create(Venda venda) throws PreexistingEntityException, Exception {
        if (venda.getVendaPK() == null) {
            venda.setVendaPK(new VendaPK());
        }
        venda.getVendaPK().setPedidoVendaid(venda.getPedidoVenda().getId());
        venda.getVendaPK().setProdutoId(venda.getProduto().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Produto produto = venda.getProduto();
            if (produto != null) {
                produto = em.getReference(produto.getClass(), produto.getId());
                venda.setProduto(produto);
            }
            PedidoVenda pedidoVenda = venda.getPedidoVenda();
            if (pedidoVenda != null) {
                pedidoVenda = em.getReference(pedidoVenda.getClass(), pedidoVenda.getId());
                venda.setPedidoVenda(pedidoVenda);
            }
            em.persist(venda);
            if (produto != null) {
                produto.getVendaCollection().add(venda);
                produto = em.merge(produto);
            }
            if (pedidoVenda != null) {
                pedidoVenda.getVendaCollection().add(venda);
                pedidoVenda = em.merge(pedidoVenda);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            if (findVenda(venda.getVendaPK()) != null) {
                throw new PreexistingEntityException("Venda " + venda + " already exists.", ex);
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void edit(Venda venda) throws NonexistentEntityException, Exception {
        venda.getVendaPK().setPedidoVendaid(venda.getPedidoVenda().getId());
        venda.getVendaPK().setProdutoId(venda.getProduto().getId());
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Venda persistentVenda = em.find(Venda.class, venda.getVendaPK());
            Produto produtoOld = persistentVenda.getProduto();
            Produto produtoNew = venda.getProduto();
            PedidoVenda pedidoVendaOld = persistentVenda.getPedidoVenda();
            PedidoVenda pedidoVendaNew = venda.getPedidoVenda();
            if (produtoNew != null) {
                produtoNew = em.getReference(produtoNew.getClass(), produtoNew.getId());
                venda.setProduto(produtoNew);
            }
            if (pedidoVendaNew != null) {
                pedidoVendaNew = em.getReference(pedidoVendaNew.getClass(), pedidoVendaNew.getId());
                venda.setPedidoVenda(pedidoVendaNew);
            }
            venda = em.merge(venda);
            if (produtoOld != null && !produtoOld.equals(produtoNew)) {
                produtoOld.getVendaCollection().remove(venda);
                produtoOld = em.merge(produtoOld);
            }
            if (produtoNew != null && !produtoNew.equals(produtoOld)) {
                produtoNew.getVendaCollection().add(venda);
                produtoNew = em.merge(produtoNew);
            }
            if (pedidoVendaOld != null && !pedidoVendaOld.equals(pedidoVendaNew)) {
                pedidoVendaOld.getVendaCollection().remove(venda);
                pedidoVendaOld = em.merge(pedidoVendaOld);
            }
            if (pedidoVendaNew != null && !pedidoVendaNew.equals(pedidoVendaOld)) {
                pedidoVendaNew.getVendaCollection().add(venda);
                pedidoVendaNew = em.merge(pedidoVendaNew);
            }
            em.getTransaction().commit();
        } catch (Exception ex) {
            String msg = ex.getLocalizedMessage();
            if (msg == null || msg.length() == 0) {
                VendaPK id = venda.getVendaPK();
                if (findVenda(id) == null) {
                    throw new NonexistentEntityException("The venda with id " + id + " no longer exists.");
                }
            }
            throw ex;
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public void destroy(VendaPK id) throws NonexistentEntityException {
        EntityManager em = null;
        try {
            em = getEntityManager();
            em.getTransaction().begin();
            Venda venda;
            try {
                venda = em.getReference(Venda.class, id);
                venda.getVendaPK();
            } catch (EntityNotFoundException enfe) {
                throw new NonexistentEntityException("The venda with id " + id + " no longer exists.", enfe);
            }
            Produto produto = venda.getProduto();
            if (produto != null) {
                produto.getVendaCollection().remove(venda);
                produto = em.merge(produto);
            }
            PedidoVenda pedidoVenda = venda.getPedidoVenda();
            if (pedidoVenda != null) {
                pedidoVenda.getVendaCollection().remove(venda);
                pedidoVenda = em.merge(pedidoVenda);
            }
            em.remove(venda);
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

    public List<Venda> findVendaEntities() {
        return findVendaEntities(true, -1, -1);
    }

    public List<Venda> findVendaEntities(int maxResults, int firstResult) {
        return findVendaEntities(false, maxResults, firstResult);
    }

    private List<Venda> findVendaEntities(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select object(o) from Venda as o");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();
        } finally {
            em.close();
        }
    }

    public List<Venda> findVenda(PedidoVenda pv) {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createNamedQuery("Venda.findByPedidoVendaid");
            q.setParameter("pedidoVendaid", pv.getId());

            return q.getResultList();

        } finally {
            em.close();
        }
    }

    public Venda findVenda(VendaPK id) {
        EntityManager em = getEntityManager();
        try {
            return em.find(Venda.class, id);
        } finally {
            em.close();
        }
    }

    public int getVendaCount() {
        EntityManager em = getEntityManager();
        try {
            Query q = em.createQuery("select count(o) from Venda as o");
            return ((Long) q.getSingleResult()).intValue();
        } finally {
            em.close();
        }
    }

}
